/*
File Name: Results.java
Purpose: Sets up the Results Page Activity, Contains function calls to the navigation drawer and to go to the Blog View page
    Also contains threads and calls to the projects database stored on CompSci02.
Related Files Used: activity_results.xml, content_results.xml, app_bar_results.xml, result_list.xml
Activities Accessed: Bookmarks.java, BlogView.java, MainActivity.java
Listed Functions: OnCreate, onBackPressed, doInBackground, onPostExecute, ProjectAdapter, getView, onNavigationItemSelected
Other Notes: This file uses an adaption of the JSON parsing and list template methods presented by Hishaam Muneer in his
    tutorial.  Please see http://www.youtube.com/user/hishamdbnd for more information on how to do this
 */

package com.example.codys.compsciproject;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;

import android.os.AsyncTask;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.example.codys.compsciproject.models.ProjectModel;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class Results extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener{

    private ListView ResultList;
    private int search_yearstart;
    private int search_yearend;
    private String search_student;
    private String search_topic;
    private String search_keyword;
    private boolean dead_file;

    /*
    Function Name: OnCreate
    Purpose: Initializes the activity.  Also, if the internal database does not exist, create it
    Function Type: Void
    Parameters Used: Bundle savedInstanceState (Refers to any data that was sent to this activity)
    Related Files: content_results.xml, activity_results.xml, app_bar_results.xml, nav_bar.xml
    Other Notes:
    */
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        // All we need to do is call activity_results, because it contains a call to content_results and app_bar_results as well
        setContentView(R.layout.activity_results);

        // Set up the toolbar at the top of the screen
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Grab the navigation drawer to apply open and close functions to it
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);

        // Sets up the navigation drawer to detect when the user taps on the hamburger (three bars in the top left corner)
        if(drawer != null)
            drawer.addDrawerListener(toggle);

        // Sets the item click events for the navigation drawer
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        if(navigationView != null)
            navigationView.setNavigationItemSelectedListener(this);

        // Allows the drawer to be opened and closed
        toggle.syncState();

        // Grabs the list from content_results.xml
        ResultList = (ListView)findViewById(R.id.ProjectResults);

        // Get the bundle sent to it by MainActivity and load it into the variables
        Bundle fromMain = getIntent().getExtras();
        search_yearstart = fromMain.getInt("SEARCH_YEARSTART");
        search_yearend = fromMain.getInt("SEARCH_YEAREND");
        search_student = fromMain.getString("SEARCH_STUDENT");
        search_topic = fromMain.getString("SEARCH_TOPIC");
        search_keyword = fromMain.getString("SEARCH_KEYWORD");

        dead_file = false;  // Indicates if results were able to be retrieved from the database

        // Execute new JSON thread, sending the url to it
        new JSONTask().execute("http://compsci02.snc.edu/cs460/2016/schocm/Project/database.txt");
    }

    /*
    Function Name: onBackPressed
    Purpose: If the navigation drawer is open, the back button will allow it to be closed
    Function Type: Void
    Parameters Used: None
    Related Files: activity_results.xml, app_bar_results.xml, nav_bar.xml
    Other Notes: None
    */
    @Override
    public void onBackPressed()
    {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if(drawer != null) {
            if (drawer.isDrawerOpen(GravityCompat.START)) {
                drawer.closeDrawer(GravityCompat.START);
            } else {
                super.onBackPressed();
            }
        }
    }

    /*
    Class Name: JSONTask
    Purpose: Contains the foreground and background threads for the database accessing, parsing, and list loading
    Related Files: content_blog_view.xml, activity_blog_view.xml, result_list.xml, database.txt (on CompSci02)
    */
    public class JSONTask extends AsyncTask<String, String, List<ProjectModel> >
    {
        /*
        Function Name: doInBackground
        Purpose: Runs before PostExecute, accesses the external project database held on CompSci02 and parses it
        Function Type: List<ProjectModel>
        Parameters Used: String... params (Contains the parameters sent to it by the AsyncTask)
        Related Files: content_results.xml, activity_results.xml, result_list.xml
        Other Notes: The function type is List<ProjectModel> because it returns the filtered ProjectModel list to the
            foreground thread.  AsyncTask is the Android method used to establish a foreground and background thread.
        */
        @Override
        protected List<ProjectModel> doInBackground(String... params)
        {
            //HttpURLConnection is what Android uses to connect to an external source
            HttpURLConnection connection = null;
            BufferedReader reader = null;

            try
            {
                // Set up and connect to the url we defined
                URL url = new URL(params[0]);
                connection = (HttpURLConnection) url.openConnection();
                connection.connect();

                // Set up the stream to get data from the database
                InputStream stream = connection.getInputStream();
                reader = new BufferedReader(new InputStreamReader(stream));
                StringBuilder buffer = new StringBuilder();
                String line;

                // As long as there is information in database.txt, read it in
                while ((line = reader.readLine()) != null)
                {
                    buffer.append(line);
                }

                // Convert the information we read in from database.txt into a string
                String finalJson = buffer.toString();

                // Convert the information into JSON
                JSONObject parentObject = new JSONObject(finalJson);

                // Split the JSON information up into an array
                JSONArray parentArray = parentObject.getJSONArray("projects");

                // FoundProjects is what will be returned to the foreground thread
                List<ProjectModel> FoundProjects = new ArrayList<>();
                ProjectModel current;

                // Go through each item in the JSON Array
                for(int i=0; i<parentArray.length(); i++)
                {
                    // Load the current JSON object into ProjectModel
                    JSONObject finalObject = parentArray.getJSONObject(i);  // Grabs all of the variables with each project entry
                    current = new ProjectModel();
                    current.setAuthor(finalObject.getString("author"));
                    current.setYear(finalObject.getInt("year"));
                    current.setTopic(finalObject.getString("topic"));
                    current.setUrl(finalObject.getString("url"));
                    current.setKeywords(finalObject.getString("keywords"));

                    // Take the search terms from MainActivity and see if we should add the current ProjectModel to
                    //  the list to send to the foreground thread.
                    if(current.getTopic().toLowerCase().contains(search_topic.toLowerCase()) || search_topic.equals(" "))
                    {
                        if(current.getAuthor().toLowerCase().contains(search_student.toLowerCase()) || search_student.equals(" "))
                        {
                            if(current.getKeywords().toLowerCase().contains(search_keyword.toLowerCase()) || search_keyword.equals(" "))
                            {
                                if(current.getYear() <= search_yearend && current.getYear() >= search_yearstart)
                                {
                                    FoundProjects.add(current);
                                }
                            }
                        }
                    }
                }

                return FoundProjects;
            }
            catch (IOException | JSONException e)
            {
                e.printStackTrace();
            }
            finally
            {
                // Close any remaining open streams
                if(connection != null)
                    connection.disconnect();

                try
                {
                    if(reader != null)
                        reader.close();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }

            // The null is returned only if we failed to open the file
            dead_file = true;
            return null;
        }

        /*
        Function Name: onPostExecute
        Purpose: Runs after the thread parsing the projects JSON database is done.  Calls ProjectAdapter and
            loads the resulting item entries into the list in content_results.xml.  Also sets the onClick
            event for each of the items in the list.
        Function Type: Void
        Parameters Used: List<ProjectModel> result (Contains the parsed and filtered results from the background thread)
        Related Files: content_results.xml, activity_results.xml, result_list.xml
        Other Notes: This function runs when the background thread is done.
        */
        @Override
        protected void onPostExecute(final List<ProjectModel> result)
        {
            super.onPostExecute(result);

            if(dead_file)
            {
                Toast.makeText(getApplicationContext(), "There are no sites to display",
                        Toast.LENGTH_LONG).show();
                return;
            }

            // Call ProjectAdapter and set up the list with the result_list.xml template and the project information
            ProjectAdapter adapter = new ProjectAdapter(getApplicationContext(), R.layout.result_list, result);
            ResultList.setAdapter(adapter);
            ResultList.setOnItemClickListener(new AdapterView.OnItemClickListener()
            {
                /*
                Function Name: onItemClick
                Purpose: Bundles the information for each project with its corresponding list item, and sets the call to BlogView
                    for each of the items
                Function Type: Void
                Parameters Used: AdapterView<?> parent (). View x (), int position, long id
                Related Files: content_results.xml, activity_results.xml, result_list.xml
                Other Notes: This function does run on every item in the list in content_results.xml
                */
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id)
                {
                    // Get the project at the listed position
                    ProjectModel thisProject = result.get(position);
                    Intent intent = new Intent(Results.this, BlogView.class);
                    Bundle BlogBund = new Bundle();

                    // Bundle all of the project information with the given list item
                    BlogBund.putString("LIST_PROJECT", thisProject.getTopic());
                    BlogBund.putString("LIST_URL", thisProject.getUrl());
                    BlogBund.putInt("LIST_YEAR", thisProject.getYear());
                    BlogBund.putString("LIST_NAME", thisProject.getAuthor());

                    // Send the project information on to BlogView
                    intent.putExtras(BlogBund);
                    startActivity(intent);
                }
            });
        }
    }

    /*
    Class Name: ProjectAdapter
    Purpose: Contains the ProjectAdapter definition, and is responsible for loading information into the
        list in content_results
    Related Files: content_results.xml, activity_results.xml, result_list.xml
    Other Notes: To help the reader understand what an ArrayAdapter is, when a list of scrollable items is
        displayed, such as the one we have in this activity, the data is populated into it using an adapter.
        The ArrayAdapter utilizes ListView and ArrayList in order to populate a list with a defined data
        source or template.  In this case, it is the template defined in result_list.xml
    */
    public class ProjectAdapter extends ArrayAdapter<ProjectModel>
    {
        public List<ProjectModel> AllProjects;  // The list of projects
        private int resource;                   // The list item index
        private LayoutInflater inflater;        // Contains the custom list item template we defined

        public ProjectAdapter(Context context, int resource, List<ProjectModel> objects)
        {
            super(context, resource, objects);
            AllProjects = objects;
            this.resource = resource;
            inflater = (LayoutInflater)getSystemService(LAYOUT_INFLATER_SERVICE);
        }

        /*
        Function Name: getView
        Purpose: Takes the results of the JSON parsing, loads it into the template defined in result_list, and loads
            a copy into each slot of the list in content_results.xml
        Function Type: View
        Parameters Used: int position (The index in the list in content_results.xml), view convertView
            (contains result_list.xml), ViewGroup parent (contains a reference to the main view for
            Results, and all sub references including the one to result_list.xml)
        Related Files: result_list.xml, activity_results.xml, content_results.xml
        Other Notes: While it contains no for loop, it executes on every item within AllProjects
        */
        @Override
        public View getView(int position, View convertView, ViewGroup parent)
        {
            if(convertView == null)
                convertView = inflater.inflate(resource, null);

            // Text fields from result_list.xml
            TextView project_title;
            TextView student_name;
            TextView year_value;

            // Grab each of the fields in result_list.xml
            project_title = (TextView)convertView.findViewById(R.id.project_title);
            student_name = (TextView)convertView.findViewById(R.id.student_name);
            year_value = (TextView)convertView.findViewById(R.id.grad_year);

            // Take the information from the projects and load them into the fields in result_list.xml
            if(project_title != null)
                project_title.setText(AllProjects.get(position).getTopic());

            if(student_name != null)
                student_name.setText(AllProjects.get(position).getAuthor());

            if(year_value != null)
                year_value.setText(String.valueOf(AllProjects.get(position).getYear()));

            // Send the project's version of result_list back to onPostExecute
            return convertView;
        }
    }


    /*
    Function Name: onNavigationItemSelected
    Purpose: Sets up Navigation Drawer item click events for the current activity
    Function Type: Boolean
    Parameters Used: MenuItem item (Refers to the navigation drawer that item that was selected)
    Related Files: nav_bar.xml, activity_results.xml, app_bar_results.xml
    Other Notes: The "Statement with empty body" warning does not cause any problems.  Thus, the
        warning is suppressed using the SuppressWarnings override at the beginning.
    */
    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item)
    {
        // Gets the id of the menu item selected
        int id = item.getItemId();

        if (id == R.id.nav_search)
        {
            // If "Home" was selected, call the activity
            Intent search_page = new Intent(this, MainActivity.class);
            startActivity(search_page);
        }
        else if (id == R.id.nav_bookmarks)
        {
            // If "Bookmarks" was selected, call the activity
            Intent bookmark_page = new Intent(this, Bookmarks.class);
            startActivity(bookmark_page);
        }
        else if (id == R.id.nav_about)
        {
            // If "About" was selected, display the toast containing the app information
            Toast.makeText(getApplicationContext(), R.string.about_string,
                    Toast.LENGTH_LONG).show();
        }

        // Grab the drawer itself
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        // As long as the drawer exists, close it
        if(drawer != null)
            drawer.closeDrawer(GravityCompat.START);

        return true;
    }
}
